SystemOrganization addCategory: #'PCube-Rendering'!
SystemOrganization addCategory: #'PCube-BasicElement'!
SystemOrganization addCategory: #'PCube-Model'!
SystemOrganization addCategory: #'PCube-Example'!

Object subclass: #PCamera
	instanceVariableNames: 'position frontVector topVector rightVector fieldOfView film'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PCube-Rendering'!

!PCamera methodsFor: 'developing' stamp: 'hjo 5/29/2012 11:07'!
capture: aPLineSegment 

	aPLineSegment captureOn: self.! !

!PCamera methodsFor: 'developing' stamp: 'hjo 5/28/2012 20:55'!
createFilmOfExtent: aPoint

	film := PFilm extent: aPoint.! !

!PCamera methodsFor: 'accessing' stamp: 'hjo 5/28/2012 17:36'!
fieldOfView: aNumber 

	fieldOfView := aNumber.! !

!PCamera methodsFor: 'accessing' stamp: 'hjo 5/28/2012 21:54'!
film

	^ film.! !

!PCamera methodsFor: 'morph' stamp: 'hjo 5/28/2012 21:02'!
filmFormMorph

	^film form asMorph.! !

!PCamera methodsFor: 'initialize-release' stamp: 'hjo 5/29/2012 09:48'!
initialize

	super initialize.
	fieldOfView := 0.4.! !

!PCamera methodsFor: 'state changing' stamp: 'hjo 5/30/2012 17:49'!
lookAt: aPPoint

	frontVector := (aPPoint - position) unitVector.
	rightVector := (frontVector crossProduct: (PPoint x: 0.0 y:0.0 z:1.0)) unitVector.
	topVector := (rightVector crossProduct: frontVector) unitVector.! !

!PCamera methodsFor: 'state changing' stamp: 'hjo 5/30/2012 17:49'!
lookOrigin

	self lookAt: (PPoint x: 0.0 y: 0.0 z: 0.0).! !

!PCamera methodsFor: 'accessing' stamp: 'hjo 5/24/2012 15:45'!
position: aPPoint 

	position := aPPoint! !

!PCamera methodsFor: 'geometry' stamp: 'hjo 5/30/2012 17:49'!
projectionPointOf3DPoint: aPPoint 

	| relPoint depth x y |
	position ifNil: [self error: 'camera position is nil'].
	frontVector ifNil: [self error: 'front vector is nil'].
	topVector ifNil: [self error: 'top vector is nil'].
	rightVector ifNil: [self error: 'right vector is nil'].
	relPoint := aPPoint - position.
	depth := relPoint dotProduct: frontVector.
	depth <= 0.0 ifTrue: [ self error: 'cannot compute because of non-positive depth'. ].
	x := relPoint dotProduct: rightVector.
	y := relPoint dotProduct: topVector.
	^PProjectedPoint new projectionPoint: (x@y) / (depth raisedTo: fieldOfView); depth: depth.! !

Object subclass: #PCube
	instanceVariableNames: 'piece1 piece2 piece3 piece4 piece5 piece6 piece7 piece8'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PCube-Model'!

!PCube methodsFor: 'as yet unclassified' stamp: 'hjo 5/24/2012 12:40'!
frontTopLeftPiece

	^PPiece new! !

!PCube methodsFor: 'as yet unclassified' stamp: 'hjo 5/24/2012 12:40'!
frontTopRightPiece
	
	^PPiece new.! !

Object subclass: #PDepthedColor
	instanceVariableNames: 'depth color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PCube-Rendering'!

!PDepthedColor classSide methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 14:53'!
depth: aFloat color: aColor 

	^self new
		depth: aFloat;
		color: aColor.! !

!PDepthedColor methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 15:44'!
color

	^ color! !

!PDepthedColor methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 14:53'!
color: aColor 

	color := aColor.! !

!PDepthedColor methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 14:59'!
depth

	^ depth! !

!PDepthedColor methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 14:53'!
depth: aFloat 

	depth := aFloat.! !

Object subclass: #PExample
	instanceVariableNames: 'c b i'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PCube-Example'!

!PExample methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 15:37'!
initialize

	super initialize.
c := PCamera new.
c fieldOfView: 0.3.
c position: (PPoint x: 2 y: 3 z: 4).  
c lookOrigin.
c createFilmOfExtent: 200@150.  
i := c filmFormMorph.
i openInWindowLabeled: 'film'.

b := InteractiveBisect new.
b open.
b inputNumber: 1.
b treatTheNumberTooSmall.
b inputNumber: 10. 
b treatTheNumberTooBig. 

b controllingBlock: [:n| ].    ! !

!PExample methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 16:37'!
main

c film clear.
c capture: PPiece new.
c film developForm.
i changed.
! !

Object subclass: #PExposing
	instanceVariableNames: 'camera'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PCube-Rendering'!

!PExposing methodsFor: 'accessing' stamp: 'hjo 5/30/2012 16:24'!
camera
	^ camera! !

!PExposing methodsFor: 'accessing' stamp: 'hjo 5/30/2012 16:24'!
camera: anObject
	camera := anObject! !

!PExposing methodsFor: 'as yet unclassified' stamp: 'hjo 5/30/2012 17:32'!
exposeLineSegmentExposedVertex: aPExposedVertex1 exposedVertex: aPExposedVertex2 

	(aPExposedVertex1 formPointIsNearWith: aPExposedVertex2)
		ifTrue: 
			[aPExposedVertex1 expose.
			aPExposedVertex2 expose.]
		ifFalse: 
			[ | m |
			m := aPExposedVertex1 middle: aPExposedVertex2.
			self exposeLineSegmentExposedVertex: aPExposedVertex1 exposedVertex: m.
			self exposeLineSegmentExposedVertex: aPExposedVertex2 exposedVertex: m.
			].! !

!PExposing methodsFor: 'as yet unclassified' stamp: 'hjo 5/30/2012 17:36'!
exposeLineSegmentVertex: aPVertex1 vertex: aPVertex2 

	self exposeLineSegmentExposedVertex: (PVertexExposing camera: camera vertex: aPVertex1) exposedVertex: (PVertexExposing camera: camera vertex: aPVertex2).! !

!PExposing methodsFor: 'as yet unclassified' stamp: 'hjo 5/30/2012 16:50'!
exposeTriangleExposedVertex: aPExposedVertex1 exposedVertex: aPExposedVertex2 exposedVertex: aPExposedVertex3 

	((aPExposedVertex1 formPointIsNearWith: aPExposedVertex2) and: [(aPExposedVertex2 formPointIsNearWith: aPExposedVertex3) and: [(aPExposedVertex3 formPointIsNearWith: aPExposedVertex1)]]) 
		ifTrue:
			[aPExposedVertex1 expose.
			aPExposedVertex2 expose.
			aPExposedVertex3 expose.]
		ifFalse:
			[ | m12 m23 m31 |
			m12 := aPExposedVertex1 middle: aPExposedVertex2.
			m23 := aPExposedVertex2 middle: aPExposedVertex3.
			m31 := aPExposedVertex3 middle: aPExposedVertex1.
			self exposeTriangleExposedVertex: m12 exposedVertex: m23 exposedVertex: m31.
			self exposeTriangleExposedVertex: aPExposedVertex1 exposedVertex: m12 exposedVertex: m31.
			self exposeTriangleExposedVertex: aPExposedVertex2 exposedVertex: m23 exposedVertex: m12.
			self exposeTriangleExposedVertex: aPExposedVertex3 exposedVertex: m31 exposedVertex: m23.].! !

!PExposing methodsFor: 'as yet unclassified' stamp: 'hjo 5/30/2012 17:36'!
exposeTriangleVertex: aPVertex1 vertex: aPVertex2 vertex: aPVertex3 

	self exposeTriangleExposedVertex: (PVertexExposing camera: camera vertex: aPVertex1) exposedVertex:  (PVertexExposing camera: camera vertex: aPVertex2) exposedVertex:  (PVertexExposing camera: camera vertex: aPVertex3).! !

Object subclass: #PFilm
	instanceVariableNames: 'pixelMatrix form'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PCube-Rendering'!

!PFilm classSide methodsFor: 'as yet unclassified' stamp: 'hjo 5/28/2012 20:35'!
extent: aPoint 

	^self new createFilmOfExtent: aPoint.! !

!PFilm methodsFor: 'develop' stamp: 'hjo 5/29/2012 14:58'!
clear

	pixelMatrix := Matrix rows:  form extent y columns:  form extent x tabulate: self tabulatingBlockForPixelMatrix.! !

!PFilm methodsFor: 'initialize-release' stamp: 'hjo 5/29/2012 14:58'!
createFilmOfExtent: aPoint 

	form := (Form extent: aPoint depth: 32) fillWhite.
	pixelMatrix := Matrix rows: aPoint y columns: aPoint x tabulate: self tabulatingBlockForPixelMatrix.
	! !

!PFilm methodsFor: 'develop' stamp: 'hjo 5/29/2012 15:44'!
developForm

	form fillWhite.
	pixelMatrix withIndicesDo: [:minClub :rowIndex :columnIndex| 
		minClub isEmpty
			ifFalse: 
				[
					form colorAt: columnIndex @ rowIndex put: minClub members anyOne color.
				]
		].! !

!PFilm methodsFor: 'expose' stamp: 'hjo 5/30/2012 17:50'!
exposeFormPoint: aPoint depth: aFloat color: aColor 

	aFloat > 0.0 ifTrue:
		[
		((aPoint y between: 1 and: pixelMatrix rowCount) and: [aPoint x between: 1 and: pixelMatrix columnCount])
			ifTrue: 
				[(pixelMatrix at: aPoint y at: aPoint x) put: (PDepthedColor depth: aFloat color: aColor).].
		].! !

!PFilm methodsFor: 'accessing' stamp: 'hjo 5/28/2012 20:46'!
form

	^form.! !

!PFilm methodsFor: 'geometry' stamp: 'hjo 5/28/2012 20:57'!
formPositionWithFormRatio: aPoint
	
	^ (form extent * (aPoint / 2)) truncated + (form extent / 2) truncated.! !

!PFilm methodsFor: 'develop' stamp: 'hjo 5/29/2012 14:58'!
tabulatingBlockForPixelMatrix

	^[:rowIndex :columnIndex| MinClub new evaluator: [:depthedColor| depthedColor depth ]].! !

Object subclass: #PLineSegment
	instanceVariableNames: 'vertex1 vertex2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PCube-BasicElement'!

!PLineSegment classSide methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 10:14'!
point1: aPPoint point2: aPPoint2 

	^self new point1: aPPoint ; point2: aPPoint2.! !

!PLineSegment classSide methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 10:42'!
point: aPPoint delta: deltaPPoint

	^self point1: aPPoint point2: (aPPoint + deltaPPoint).! !

!PLineSegment classSide methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 10:45'!
point: aPPoint deltaX: aNumber 

	^self point: aPPoint delta: (PPoint x:aNumber y:0 z:0).! !

!PLineSegment classSide methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 10:43'!
point: aPPoint deltaY: aNumber 

	^self point: aPPoint delta: (PPoint x:0 y:aNumber z:0).! !

!PLineSegment classSide methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 10:43'!
point: aPPoint deltaZ: aNumber

	^self point: aPPoint delta: (PPoint x:0 y:0 z:aNumber).! !

!PLineSegment methodsFor: 'as yet unclassified' stamp: 'hjo 5/30/2012 17:34'!
captureOn: aPCamera

	PExposing new
		camera: aPCamera;
		exposeLineSegmentVertex: vertex1 vertex: vertex2.! !

!PLineSegment methodsFor: 'accessing' stamp: 'hjo 5/29/2012 14:20'!
point1
	^ vertex1 point.! !

!PLineSegment methodsFor: 'accessing' stamp: 'hjo 5/29/2012 14:20'!
point1: aPPoint
	vertex1 := PVertex new point: aPPoint; color: Color blue.
! !

!PLineSegment methodsFor: 'accessing' stamp: 'hjo 5/29/2012 14:20'!
point2
	^ vertex2 point.! !

!PLineSegment methodsFor: 'accessing' stamp: 'hjo 5/29/2012 14:21'!
point2: aPPoint
	vertex2 := PVertex new point: aPPoint; color: Color red.
! !

!PLineSegment methodsFor: 'accessing' stamp: 'hjo 5/29/2012 14:19'!
vertex1
	^ vertex1! !

!PLineSegment methodsFor: 'accessing' stamp: 'hjo 5/29/2012 14:19'!
vertex1: aPVertex
	vertex1 := aPVertex
! !

!PLineSegment methodsFor: 'accessing' stamp: 'hjo 5/29/2012 14:19'!
vertex2
	^ vertex2! !

!PLineSegment methodsFor: 'accessing' stamp: 'hjo 5/29/2012 14:19'!
vertex2: aPVertex
	vertex2 := aPVertex
! !

Object subclass: #PPiece
	instanceVariableNames: 'vertices edges triangles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PCube-Model'!

!PPiece methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 17:34'!
captureOn: aPCamera 

	triangles do: [:each| aPCamera capture: each ].
	edges do: [:each| aPCamera capture: each ].! !

!PPiece methodsFor: 'as yet unclassified' stamp: 'hjo 5/30/2012 00:43'!
initialize

	super initialize.
	self initializeVertices.
	self initializeEdges.
	self initializeTriangles.
! !

!PPiece methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 16:35'!
initializeEdges

	| stream |
	edges := Array new: 12.
	stream := WriteStream on: edges.

	stream nextPut: (PLineSegment new vertex1: self x1y1z1; vertex2: self x2y1z1).
	stream nextPut: (PLineSegment new vertex1: self x1y1z2; vertex2: self x2y1z2).
	stream nextPut: (PLineSegment new vertex1: self x1y2z1; vertex2: self x2y2z1).
	stream nextPut: (PLineSegment new vertex1: self x1y2z2; vertex2: self x2y2z2).
	
	stream nextPut: (PLineSegment new vertex1: self x1y1z1; vertex2: self x1y2z1).
	stream nextPut: (PLineSegment new vertex1: self x1y1z2; vertex2: self x1y2z2).
	stream nextPut: (PLineSegment new vertex1: self x2y1z1; vertex2: self x2y2z1).
	stream nextPut: (PLineSegment new vertex1: self x2y1z2; vertex2: self x2y2z2).
	
	stream nextPut: (PLineSegment new vertex1: self x1y1z1; vertex2: self x1y1z2).
	stream nextPut: (PLineSegment new vertex1: self x1y2z1; vertex2: self x1y2z2).
	stream nextPut: (PLineSegment new vertex1: self x2y1z1; vertex2: self x2y1z2).
	stream nextPut: (PLineSegment new vertex1: self x2y2z1; vertex2: self x2y2z2).
! !

!PPiece methodsFor: 'as yet unclassified' stamp: 'hjo 5/30/2012 15:46'!
initializeTriangles

	| stream |
	stream := WriteStream on: Array new.

	stream nextPut: (PTriangle vertex1: self x1y2z2 vertex2: self x2y1z2 vertex3: self x2y2z2).
	stream nextPut: (PTriangle vertex1: self x1y2z2 vertex2: self x2y1z2 vertex3: self x1y1z2).

	stream nextPut: (PTriangle vertex1: self x1y2z1 vertex2: self x2y1z1 vertex3: self x2y2z1).
	stream nextPut: (PTriangle vertex1: self x1y2z1 vertex2: self x2y1z1 vertex3: self x1y1z1).

	stream nextPut: (PTriangle vertex1: self x1y1z1 vertex2: self x1y1z2 vertex3: self x1y2z1).
	stream nextPut: (PTriangle vertex1: self x1y2z1 vertex2: self x1y1z2 vertex3: self x1y2z2).

	stream nextPut: (PTriangle vertex1: self x2y1z1 vertex2: self x2y1z2 vertex3: self x2y2z1).
	stream nextPut: (PTriangle vertex1: self x2y2z1 vertex2: self x2y1z2 vertex3: self x2y2z2).

	stream nextPut: (PTriangle vertex1: self x1y1z1 vertex2: self x2y1z1 vertex3: self x1y1z2).
	stream nextPut: (PTriangle vertex1: self x2y1z1 vertex2: self x1y1z2 vertex3: self x2y1z2).

	stream nextPut: (PTriangle vertex1: self x1y2z1 vertex2: self x2y2z1 vertex3: self x1y2z2).
	stream nextPut: (PTriangle vertex1: self x2y2z1 vertex2: self x1y2z2 vertex3: self x2y2z2).
	
	triangles := stream contents.
! !

!PPiece methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 16:27'!
initializeVertices

	| stream |
	vertices := Array new: 8.
	stream := WriteStream on: vertices.
	stream nextPut: (PVertex new point: (PPoint x: -1 y: -1 z: -1); color: Color green).
	stream nextPut: (PVertex new point: (PPoint x: -1 y: -1 z: 1); color: Color red).
	stream nextPut: (PVertex new point: (PPoint x: -1 y: 1 z: -1); color: Color blue).
	stream nextPut: (PVertex new point: (PPoint x: -1 y: 1 z: 1); color: Color cyan).
	stream nextPut: (PVertex new point: (PPoint x: 1 y: -1 z: -1); color: Color darkGray).
	stream nextPut: (PVertex new point: (PPoint x: 1 y: -1 z: 1); color: Color lightBlue).
	stream nextPut: (PVertex new point: (PPoint x: 1 y: 1 z: -1); color: Color orange).
	stream nextPut: (PVertex new point: (PPoint x: 1 y: 1 z: 1); color: Color tan).
! !

!PPiece methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 16:18'!
x1y1z1

	^vertices at: 1.! !

!PPiece methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 16:18'!
x1y1z2

	^vertices at: 2.! !

!PPiece methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 16:18'!
x1y2z1

	^vertices at: 3.! !

!PPiece methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 16:18'!
x1y2z2

	^vertices at: 4.! !

!PPiece methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 16:19'!
x2y1z1

	^vertices at: 5.! !

!PPiece methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 16:19'!
x2y1z2

	^vertices at: 6.! !

!PPiece methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 16:19'!
x2y2z1

	^vertices at: 7.! !

!PPiece methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 16:19'!
x2y2z2

	^vertices at: 8.! !

Object subclass: #PPiecePlane
	instanceVariableNames: 'normalVector constantNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PCube-Model'!

Object subclass: #PPoint
	instanceVariableNames: 'x y z'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PCube-Rendering'!

!PPoint classSide methodsFor: 'as yet unclassified' stamp: 'hjo 5/24/2012 15:44'!
x: aSmallInteger y: aSmallInteger2 z: aSmallInteger3 

	^self new 
		x: aSmallInteger;
		y: aSmallInteger2;
		z: aSmallInteger3
	! !

!PPoint methodsFor: 'arithmetic' stamp: 'hjo 5/29/2012 10:38'!
+ aPPoint 
	
	^PPoint x: (x + aPPoint x) y: (y + aPPoint y) z: (z + aPPoint z).! !

!PPoint methodsFor: 'arithmetic' stamp: 'hjo 5/24/2012 15:50'!
- aPPoint 

	^PPoint x: (x - (aPPoint x)) y: (y - (aPPoint y)) z: (z - (aPPoint z)).! !

!PPoint methodsFor: 'cross' stamp: 'hjo 5/24/2012 16:00'!
crossProduct: aPPoint 

	^PPoint x: (self xOfCrossProduct: aPPoint) y: (self yOfCrossProduct: aPPoint) z: (self zOfCrossProduct: aPPoint).! !

!PPoint methodsFor: 'arithmetic' stamp: 'hjo 5/24/2012 16:13'!
dotProduct: aPPoint 

	^(x * aPPoint x) + (y * aPPoint y) + (z * aPPoint z).! !

!PPoint methodsFor: 'length' stamp: 'hjo 5/24/2012 15:56'!
length

	^self squaredLength sqrt.! !

!PPoint methodsFor: 'as yet unclassified' stamp: 'hjo 5/30/2012 17:51'!
middle: aPPoint 

	^PPoint x: (x + aPPoint x)/2.0 y: (y + aPPoint y)/2.0 z: (z + aPPoint z)/2.0.! !

!PPoint methodsFor: 'printing' stamp: 'hjo 5/24/2012 16:06'!
printOn: aStream

	aStream nextPut: $(.
	aStream print: x.
	aStream nextPut: $,.
	aStream print: y.
	aStream nextPut: $,.
	aStream print: z.
	aStream nextPut: $).! !

!PPoint methodsFor: 'arithmetic' stamp: 'hjo 5/24/2012 15:57'!
scaledBy: aFloat 

	^PPoint x: (x * aFloat) y: (y * aFloat) z: (z * aFloat).! !

!PPoint methodsFor: 'length' stamp: 'hjo 5/24/2012 15:56'!
squaredLength

	^((x squared) + (y squared) + (z squared))! !

!PPoint methodsFor: 'length' stamp: 'hjo 5/24/2012 15:55'!
unitVector

	^self scaledBy: self length reciprocal.! !

!PPoint methodsFor: 'accessing' stamp: 'hjo 5/24/2012 15:50'!
x

	^ x! !

!PPoint methodsFor: 'accessing' stamp: 'hjo 5/24/2012 15:45'!
x: aSmallInteger 

	x := aSmallInteger! !

!PPoint methodsFor: 'cross' stamp: 'hjo 5/24/2012 16:01'!
xOfCrossProduct: aPPoint 

	^((y * aPPoint z) - (z * aPPoint y)).! !

!PPoint methodsFor: 'accessing' stamp: 'hjo 5/24/2012 15:50'!
y

	^ y! !

!PPoint methodsFor: 'accessing' stamp: 'hjo 5/24/2012 15:45'!
y: aSmallInteger 

	y := aSmallInteger! !

!PPoint methodsFor: 'cross' stamp: 'hjo 5/24/2012 16:02'!
yOfCrossProduct: aPPoint 

	^((z * aPPoint x) - (x * aPPoint z)).! !

!PPoint methodsFor: 'accessing' stamp: 'hjo 5/24/2012 15:50'!
z

	^ z! !

!PPoint methodsFor: 'accessing' stamp: 'hjo 5/24/2012 15:45'!
z: aSmallInteger 

	z := aSmallInteger! !

!PPoint methodsFor: 'cross' stamp: 'hjo 5/24/2012 16:02'!
zOfCrossProduct: aPPoint 

	^((x * aPPoint y) - (y * aPPoint x)).! !

Object subclass: #PProjectedPoint
	instanceVariableNames: 'projectionPoint depth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PCube-Rendering'!

!PProjectedPoint methodsFor: 'accessing' stamp: 'hjo 5/28/2012 17:35'!
depth
	^ depth! !

!PProjectedPoint methodsFor: 'accessing' stamp: 'hjo 5/28/2012 17:36'!
depth: aNumber
	depth := aNumber! !

!PProjectedPoint methodsFor: 'accessing' stamp: 'hjo 5/28/2012 17:35'!
projectionPoint
	^ projectionPoint! !

!PProjectedPoint methodsFor: 'accessing' stamp: 'hjo 5/28/2012 17:36'!
projectionPoint: aPoint
	projectionPoint := aPoint! !

Object subclass: #PTriangle
	instanceVariableNames: 'vertex1 vertex2 vertex3'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PCube-BasicElement'!

!PTriangle classSide methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 17:32'!
vertex1: aPVertex vertex2: aPVertex2 vertex3: aPVertex3 

	^self new
		vertex1: aPVertex;
		vertex2: aPVertex2;
		vertex3: aPVertex3 ! !

!PTriangle methodsFor: 'as yet unclassified' stamp: 'hjo 5/30/2012 17:34'!
captureOn: aPCamera 

	PExposing new
		camera: aPCamera;
		exposeTriangleVertex: vertex1 vertex: vertex2 vertex: vertex3.! !

!PTriangle methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 17:32'!
vertex1: aPVertex 

	vertex1 := aPVertex ! !

!PTriangle methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 17:32'!
vertex2: aPVertex 

	vertex2 := aPVertex ! !

!PTriangle methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 17:33'!
vertex3: aPVertex 

	vertex3 := aPVertex ! !

Object subclass: #PVertex
	instanceVariableNames: 'point color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PCube-BasicElement'!

!PVertex methodsFor: 'accessing' stamp: 'hjo 5/29/2012 11:09'!
color
	^ color! !

!PVertex methodsFor: 'accessing' stamp: 'hjo 5/29/2012 11:09'!
color: aColor
	color := aColor
! !

!PVertex methodsFor: 'initialize-release' stamp: 'hjo 5/29/2012 11:11'!
initialize

	super initialize.
	color := Color red.! !

!PVertex methodsFor: 'as yet unclassified' stamp: 'hjo 5/29/2012 23:09'!
middle: aPVertex 

	^PVertex new 
		point: (point middle: aPVertex point); 
		color: (color mix: aPVertex color shades: 3) middle.! !

!PVertex methodsFor: 'accessing' stamp: 'hjo 5/29/2012 11:09'!
point
	^ point! !

!PVertex methodsFor: 'accessing' stamp: 'hjo 5/29/2012 11:09'!
point: aPPoint
	point := aPPoint
! !

Object subclass: #PVertexExposing
	instanceVariableNames: 'camera vertex projectedPoint formPoint'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PCube-Rendering'!

!PVertexExposing classSide methodsFor: 'as yet unclassified' stamp: 'hjo 5/30/2012 16:30'!
camera: aPCamera vertex: aPVertex 

	^self new
		camera: aPCamera;
		vertex: aPVertex.! !

!PVertexExposing methodsFor: 'accessing' stamp: 'hjo 5/30/2012 16:24'!
camera
	^ camera! !

!PVertexExposing methodsFor: 'accessing' stamp: 'hjo 5/30/2012 16:41'!
camera: aPCamera
	camera := aPCamera! !

!PVertexExposing methodsFor: 'as yet unclassified' stamp: 'hjo 5/30/2012 16:48'!
expose
	camera film exposeFormPoint: self formPoint depth: self projectedPoint depth color: vertex color.! !

!PVertexExposing methodsFor: 'accessing' stamp: 'hjo 5/30/2012 16:46'!
formPoint
	formPoint ifNil: [
		formPoint := camera film formPositionWithFormRatio: self projectedPoint projectionPoint.      
		].
	^ formPoint! !

!PVertexExposing methodsFor: 'as yet unclassified' stamp: 'hjo 5/30/2012 17:57'!
formPointIsNearWith: aPExposedVertex 

	| d |
	d := self formPoint - aPExposedVertex formPoint.
	^(d dotProduct: d) <= 2.! !

!PVertexExposing methodsFor: 'as yet unclassified' stamp: 'hjo 5/30/2012 16:47'!
middle: aPExposedVertex 

	^self class camera: camera vertex:  (vertex middle: aPExposedVertex vertex).! !

!PVertexExposing methodsFor: 'accessing' stamp: 'hjo 5/30/2012 16:43'!
projectedPoint
	projectedPoint ifNil: [
		projectedPoint := camera projectionPointOf3DPoint: vertex point.
		].
	^ projectedPoint! !

!PVertexExposing methodsFor: 'accessing' stamp: 'hjo 5/30/2012 16:24'!
vertex
	^ vertex! !

!PVertexExposing methodsFor: 'accessing' stamp: 'hjo 5/30/2012 16:44'!
vertex: aPVertex
	vertex := aPVertex! !
